@@ -2,8 +2,8 @@
 
 ;; Author: Johannes Mueller <github@johannes-mueller.org>
 ;; URL: https://github.com/johannes-mueller/test-cockpit.el
-;; Package-Version: 20260217.2029
-;; Package-Revision: b4a7445b62c6
+;; Package-Version: 20260224.1810
+;; Package-Revision: 09b8978bba47
 ;; License: GPLv3
 ;; SPDX-License-Identifier: GPL-3.0-only
 ;; Package-Requires: ((emacs "28.1") (projectile "2.7") (transient "0.7.0") (toml "0.2.0"))
@@ -32,32 +32,22 @@
 ;; It is suggested that you bind the following two commands to keybindings that
 ;; suit you best.
 
-;; * `test-cockpit-repeat-test-or-projectile-test'
+;; * `test-cockpit-repeat-test'
 ;;   This should be bound to a quickly reachable keybinding, that you can find
 ;;   easily and quickly.  It tries to the last test that the current project
 ;;   has been tested with.  If in the current session the project has not been
 ;;   tested yet, a dialog is opened for you to choose the way the testing should
 ;;   be performed.
 
-;;   If the project type is not supported it falls back to
-;;   `projectile-test-project'.
-
 ;;   In either way, the test command that you give is remembered.  Next time you
 ;;   hit your key binding, the exact same test command for the project is
 ;;   repeated.
 
-;; * `test-cockpit-test-or-projectile-test'
+;; * `test-cockpit-dispatch'
 ;;   This does open the test dialog for you to setup the test command.  If the
 ;;   project type is not supported, it falls back to `projectile-test-command`.  So
 ;;   use this if you don't want to repeat the last test, but run a different one.
 
-;; You can also use the following commands to run tests in a more manual way
-
-;; * `test-cockpit-test-project' to run the whole test suite.
-;; * `test-cockpit-test-module' runs only the tests of the current module.
-;; * `test-cockpit-test-function' runs only the test of the function at point.
-;; * `test-cockpit-repeat-test' repeats exactly the *previous* test run.
-
 ;; If the current function at point or the current module cannot be determined,
 ;; the last tested module resp. last tested function are tested.  If there are no
 ;; last tests, an error message is thrown.
@@ -72,6 +62,11 @@
 
 (declare-function dape "dape" (config &optional skip-compile))
 
+(defcustom test-cockpit-fallback-test-command #'projectile-test-project
+  "Fall back function for unsupported project types."
+  :group 'test-cockpit
+  :type 'function)
+
 (defvar test-cockpit--project-types nil
   "List of known project types.
 Project types can be added to the list using
@@ -121,7 +116,8 @@ class is needed which implements the methods of the base class.")
   "Supply the function to be called when the whole project is to be tested.
 The function has to have the signature `(defun fun (_ ARGS))' where
 ARGS is the argument list passed to the test frame work."
-  nil)
+  (when test-cockpit-fallback-test-command
+    (call-interactively test-cockpit-fallback-test-command)))
 
 (cl-defmethod test-cockpit--test-module-command ((_obj test-cockpit--engine))
   "Supply the function to be called when a module is to be tested.
@@ -216,18 +212,6 @@ If no engine is yet started for the project, it will be started."
       (setf (test-cockpit--engine-for-current-project) new-engine)
       new-engine)))
 
-(defun test-cockpit--dummy-engine-p (&optional engine)
-  "Return non-nil if ENGINE or the current project enigne is a dummy engine."
-    (not (oref (or engine (test-cockpit--retrieve-engine)) project-type)))
-
-(defun test-cockpit--real-engine-or-error ()
-  "Retrieve the engine for the project; error if the project type is unsupported."
-  (let ((engine (test-cockpit--retrieve-engine)))
-    (if (test-cockpit--dummy-engine-p engine)
-        (signal "Project type %s not supported by test-cockpit or engine not installed"
-                (projectile-project-type))
-      engine)))
-
 (defun test-cockpit--make-test-command (method thing args)
   "Setup the test command to be issued.
 The supplied METHOD for the current engine is called to get the
@@ -420,8 +404,9 @@ and thus can be repeated using `test-cockpit-repeat-test'."
     (when-let* ((last-cmd (test-cockpit--last-custom-command)))
       (setq compile-command last-cmd))
     (call-interactively #'compile))
-  (oset (test-cockpit--retrieve-engine) last-custom-command compile-command)
-  (oset (test-cockpit--retrieve-engine) last-command compile-command))
+  (let ((engine (test-cockpit--retrieve-engine)))
+    (oset engine last-custom-command compile-command)
+    (oset engine last-command compile-command)))
 
 (defun test-cockpit--process-custom-command (command regex replacement)
   "Replace placeholder for an argument by the argument in a command.
@@ -457,63 +442,12 @@ If the for the project no test has been run during the current
 session, the main dispatch dialog is invoked."
   (interactive
    (list (transient-args 'test-cockpit-prefix)))
-  (if-let* ((last-cmd (oref (test-cockpit--real-engine-or-error) last-command)))
+  (if-let* ((last-cmd (oref (test-cockpit--retrieve-engine) last-command)))
       (if (eq last-cmd 'test-cockpit--last-command-was-dape)
           (test-cockpit-dape-debug-repeat-test)
         (test-cockpit--run-test last-cmd))
     (test-cockpit-dispatch)))
 
-;;;###autoload
-(defun test-cockpit-test-or-projectile-build ()
-  "Test or build the project depending on if the project type is supported.
-If the project type is supported, function
-`test-cockpit-repeat-test' is run.  Otherwise the project build
-is launched by calling the function `projectile-compile-project'."
-  (interactive)
-  (if (test-cockpit--dummy-engine-p)
-      (test-cockpit--projectile-build)
-    (test-cockpit-dispatch)))
-
-;;;###autoload
-(defun test-cockpit-repeat-test-or-projectile-build ()
-  "Repeat the last test or build action (native or projectile).
-If the project type is supported, function `test-cockpit-repeat'
-test is invoked.  That means, the last test action is repeated or
-if the project has not seen a test action during the session, the
-test menu is shown.  If the project type is unknown the last
-build command is repeated by the function
-`projectile-build-project' a prompt to type a build command is
-shown."
-  (interactive)
-  (if (test-cockpit--dummy-engine-p)
-      (test-cockpit--repeat-projectile-build)
-    (test-cockpit-repeat-test)))
-
-;;;###autoload
-(defun test-cockpit-test-or-projectile-test ()
-  "Test the project falling back projectile if project type is not supported.
-If the project type is supported, function
-`test-cockpit-repeat-test' is run.  Otherwise the project tested
-calling the function `projectile-test-project'."
-  (interactive)
-  (if (test-cockpit--dummy-engine-p)
-      (test-cockpit--projectile-test)
-    (test-cockpit-dispatch)))
-
-;;;###autoload
-(defun test-cockpit-repeat-test-or-projectile-test ()
-  "Repeat the last test action (native or projectile).
-If the project type is supported, function `test-cockpit-repeat'
-test is invoked.  That means, the last test action is repeated or
-if the project has not seen a test action during the session, the
-test menu is shown.  If the project type is unknown the last test
-command is repeated by the function `projectile-test-project' a
-prompt to type a test command is shown."
-  (interactive)
-  (if (test-cockpit--dummy-engine-p)
-      (test-cockpit--repeat-projectile-test)
-    (test-cockpit-repeat-test)))
-
 ;;;###autoload
 (defun test-cockpit-repeat-interactive-test (&optional args)
   "Repeat the last interactive test command.
@@ -583,36 +517,6 @@ Then memorize that last test was a dape session."
   (dape config)
   (oset (test-cockpit--retrieve-engine) last-command 'test-cockpit--last-command-was-dape))
 
-(defun test-cockpit--projectile-build (&optional last-cmd)
-  "Launch a projectile driven build process.
-If last executed command LAST-CMD is given the command is
-repeated as is.  Otherwise function `projectile-compile-project'
-is called interactively and the actual compile command is stored
-for repetition."
-  (if last-cmd
-      (test-cockpit--issue-compile-command last-cmd)
-    (projectile-compile-project nil)
-    (oset (test-cockpit--retrieve-engine) last-build-command compile-command)))
-
-(defun test-cockpit--repeat-projectile-build ()
-  "Repeat the last projectile driven build process."
-  (test-cockpit--projectile-build (test-cockpit--last-build-command)))
-
-(defun test-cockpit--projectile-test (&optional last-cmd)
-  "Launch a projectile driven test process.
-If last executed command LAST-CMD is given the command is
-repeated as is.  Otherwise function `projectile-test-project' is
-called interactively and the actual test command is stored for
-repetition."
-  (if last-cmd
-      (test-cockpit--issue-compile-command last-cmd)
-    (projectile-test-project nil)
-    (oset (test-cockpit--retrieve-engine) last-test-command compile-command)))
-
-(defun test-cockpit--repeat-projectile-test ()
-  "Repeat the last projectile driven test process."
-  (test-cockpit--projectile-test (test-cockpit--last-test-command)))
-
 (defun test-cockpit--current-module-string ()
   "Ask the current engine for the current module string."
   (test-cockpit--engine-current-module-string (test-cockpit--retrieve-engine)))
@@ -722,13 +626,14 @@ repetition."
     ('test-cockpit-test-function (format "function: %s" (test-cockpit--strip-project-root (test-cockpit--last-function-string))))
     (_ "")))
 
+
+;;;###autoload
 (defun test-cockpit-dispatch ()
   "Invoke the user interface of to setup and run tests.
 If the repeat suffix has been appended it is removed afterwards
 as will be no longer valid and we don't want the suffixes to
 accumulate."
   (interactive)
-  (test-cockpit--real-engine-or-error)
   (test-cockpit-prefix))
 
 (defun test-cockpit--join-filter-switches (candidates allowed)
